/*******************************************************************************
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.util;

import org.ow2.orchestra.facade.Deployment;
import org.ow2.orchestra.facade.criteria.Criteria;
import org.ow2.orchestra.facade.criteria.Field;
import org.ow2.orchestra.facade.criteria.FilterField;
import org.ow2.orchestra.facade.criteria.Order;
import org.ow2.orchestra.facade.criteria.PagedResult;
import org.ow2.orchestra.facade.criteria.SortableField;
import org.ow2.orchestra.facade.data.runtime.JobData;
import org.ow2.orchestra.facade.data.runtime.PendingMessageData;
import org.ow2.orchestra.facade.def.impl.AssignActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.CatchAllDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.CatchDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.CompensateActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.CompensateScopeActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.CompensationHandlerDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.CorrelationSetDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.EmptyActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.ExitActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.FlowActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.ForeachActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.IfActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.InvokeDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.PartnerLinkDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.PickActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.ProcessDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.ProcessStateUpdateImpl;
import org.ow2.orchestra.facade.def.impl.ReceiveActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.RepeatUntilActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.ReplyActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.RethrowActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.ScopeActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.SequenceActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.SourceImpl;
import org.ow2.orchestra.facade.def.impl.TargetImpl;
import org.ow2.orchestra.facade.def.impl.TerminationHandlerDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.ThrowActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.VariableDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.WaitActivityDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.WhileActivityDefinitionImpl;
import org.ow2.orchestra.facade.rest.ByteArrayWrapper;
import org.ow2.orchestra.facade.rest.CollectionWrapper;
import org.ow2.orchestra.facade.rest.InterfaceWrapper;
import org.ow2.orchestra.facade.rest.SimpleCollectionWrapper;
import org.ow2.orchestra.facade.runtime.impl.AssignActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.CatchActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.CatchAllActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.CompensateActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.CompensateScopeActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.CompensationHandlerActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.CorrelationSetInitializationImpl;
import org.ow2.orchestra.facade.runtime.impl.EmptyActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.ExitActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.FlowActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.ForeachActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.IfActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.InvokeActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.MessageImpl;
import org.ow2.orchestra.facade.runtime.impl.PartnerLinkUpdateImpl;
import org.ow2.orchestra.facade.runtime.impl.PickActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.ProcessInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.ProcessInstanceStateUpdateImpl;
import org.ow2.orchestra.facade.runtime.impl.ReceiveActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.RepeatUntilActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.ReplyActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.RethrowActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.ScopeActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.ScopeStateUpdateImpl;
import org.ow2.orchestra.facade.runtime.impl.SequenceActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.TerminationHandlerActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.ThrowActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.UnknownActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.VariableUpdateImpl;
import org.ow2.orchestra.facade.runtime.impl.WaitActivityInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.WaitingActivityImpl;
import org.ow2.orchestra.facade.runtime.impl.WhileActivityInstanceImpl;
import org.ow2.orchestra.util.wsdl.WsdlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import javax.wsdl.Definition;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.adapters.XmlAdapter;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Guillaume Porcher
 */
public class JAXBUtils {
  public static final Class<?>[] JAXB_CLASSES = {
          /* Wrappers */
          InterfaceWrapper.class,
          SimpleCollectionWrapper.class,
          CollectionWrapper.class,
          ByteArrayWrapper.class,
          /* Definition classes */
          AssignActivityDefinitionImpl.class,
          CatchAllDefinitionImpl.class,
          CatchDefinitionImpl.class,
          CompensateActivityDefinitionImpl.class,
          CompensateScopeActivityDefinitionImpl.class,
          CompensationHandlerDefinitionImpl.class,
          EmptyActivityDefinitionImpl.class,
          ExitActivityDefinitionImpl.class,
          FlowActivityDefinitionImpl.class,
          ForeachActivityDefinitionImpl.class,
          IfActivityDefinitionImpl.class,
          InvokeDefinitionImpl.class,
          PickActivityDefinitionImpl.class,
          ProcessDefinitionImpl.class,
          ReceiveActivityDefinitionImpl.class,
          RepeatUntilActivityDefinitionImpl.class,
          ReplyActivityDefinitionImpl.class,
          RethrowActivityDefinitionImpl.class,
          ScopeActivityDefinitionImpl.class,
          SequenceActivityDefinitionImpl.class,
          TerminationHandlerDefinitionImpl.class,
          ThrowActivityDefinitionImpl.class,
          WaitActivityDefinitionImpl.class,
          WhileActivityDefinitionImpl.class,
          VariableDefinitionImpl.class,
          PartnerLinkDefinitionImpl.class,
          ProcessStateUpdateImpl.class,
          CorrelationSetDefinitionImpl.class,
          SourceImpl.class,
          TargetImpl.class,
          /* Runtime classes */
          AssignActivityInstanceImpl.class,
          CompensateScopeActivityInstanceImpl.class,
          UnknownActivityInstanceImpl.class,
          SequenceActivityInstanceImpl.class,
          FlowActivityInstanceImpl.class,
          ForeachActivityInstanceImpl.class,
          WhileActivityInstanceImpl.class,
          RepeatUntilActivityInstanceImpl.class,
          IfActivityInstanceImpl.class,
          PickActivityInstanceImpl.class,
          CatchActivityInstanceImpl.class,
          TerminationHandlerActivityInstanceImpl.class,
          CompensationHandlerActivityInstanceImpl.class,
          CatchAllActivityInstanceImpl.class,
          ThrowActivityInstanceImpl.class,
          ReplyActivityInstanceImpl.class,
          CompensateActivityInstanceImpl.class,
          ExitActivityInstanceImpl.class,
          ReceiveActivityInstanceImpl.class,
          RethrowActivityInstanceImpl.class,
          EmptyActivityInstanceImpl.class,
          WaitActivityInstanceImpl.class,
          InvokeActivityInstanceImpl.class,
          ProcessInstanceImpl.class,
          ProcessInstanceStateUpdateImpl.class,
          ScopeActivityInstanceImpl.class,
          VariableUpdateImpl.class,
          CorrelationSetInitializationImpl.class,
          PartnerLinkUpdateImpl.class,
          ScopeStateUpdateImpl.class,
          MessageImpl.class,
          WaitingActivityImpl.class,
          /* Data objects */
          JobData.class,
          PendingMessageData.class,
          Deployment.class,
          PagedResult.class,
          Criteria.class,
          Order.class,
          SortableField.class,
          FilterField.class,
          Field.class
  };

  /**
     * @author Guillaume Porcher
     */
    public static class JAXBDOMDocumentAdapter extends XmlAdapter<String, Document> {

      public Document unmarshal(String value) {
        return XmlUtil.getDocumentFromString(value);
      }

      public String marshal(Document value) {
        return XmlUtil.toString(value);
      }

    }

  /**
   * @author Guillaume Porcher
   */
  public static class JAXBSerializableAdapter extends XmlAdapter<ByteArrayWrapper, Serializable> {

    public Serializable unmarshal(ByteArrayWrapper value) throws IOException, ClassNotFoundException {
      return Misc.deserialize(value.getContent());
    }

    public ByteArrayWrapper marshal(Serializable value) throws IOException, ClassNotFoundException {
      return ByteArrayWrapper.make(Misc.serialize(value));
    }

  }
  /**
     * @author Guillaume Porcher
     */
    public static class JAXBDOMNodeAdapter extends XmlAdapter<String, Node> {

      public Node unmarshal(String value) {
        final Document doc = XmlUtil.getDocumentFromString(value);
        if (doc != null) {
          final Element elt = doc.getDocumentElement();
          final Element child = XmlUtil.element(elt);
          if (child != null) {
            return child;
          } else {
            return elt.getFirstChild();
          }
        }
        return null;
      }

      public String marshal(Node value) {
        return XmlUtil.toString(value);
      }

    }

  /**
   * @author Guillaume Porcher
   */
  public static class JAXBDefinitionsAdapter extends XmlAdapter<SimpleCollectionWrapper<String>, Set<Definition>> {

    public Set<Definition> unmarshal(SimpleCollectionWrapper<String> value) {
      if (value != null && value.getContent() != null) {
        Set<Definition> res = new HashSet<Definition>();
        for (String wsdlAsString : value.getContent()) {
          res.add(WsdlUtil.readWsdl(XmlUtil.getDocumentFromString(wsdlAsString).getDocumentElement()));
        }
        return res;
      }
      return null;
    }

    public SimpleCollectionWrapper<String> marshal(Set<Definition> value) {
      if (value == null) {
        return SimpleCollectionWrapper.make(null);
      }
      List<String> res = new ArrayList<String>();
      for (Definition def : value) {
        res.add(WsdlUtil.writeWsdl(def));
      }
      return SimpleCollectionWrapper.make(res);
    }

  }

  /**
   * Inspired from {@link javax.xml.bind.annotation.adapters.XmlAdapter} javadoc example.
   * @author Guillaume Porcher
   */
  public static class JAXBMapAdapter<K, V, W> extends XmlAdapter<JAXBMapAdapter.InternalMapType<K, W>, Map<K, V>> {

    @XmlRootElement(name = "entry")
    public static class InternalMapEntryType<K, V> {
      @XmlElement
      public K key;
      @XmlElement
      public V value;

      public InternalMapEntryType() {
      }

      public InternalMapEntryType(K key, V value) {
        this.key = key;
        this.value = value;
      }
    }

    @XmlRootElement(name = "map")
    public static class InternalMapType<K, V> {
      @XmlElement
      private List<InternalMapEntryType<K, V>> entries = new ArrayList<InternalMapEntryType<K, V>> ();

      public List<InternalMapEntryType<K, V>> getEntries() {
        return entries;
      }
    }

    protected XmlAdapter<W, V> valueAdapter = null;

    public Map<K, V> unmarshal(InternalMapType<K, W> value) throws Exception {
      Map<K, V> resultMap = new HashMap<K, V>();
      for (InternalMapEntryType<K, W> entry : value.getEntries()) {
        if (valueAdapter == null) {
          resultMap.put(entry.key, (V) entry.value);
        } else {
          resultMap.put(entry.key, valueAdapter.unmarshal(entry.value));
        }
      }
      return resultMap;
    }

    public InternalMapType<K,W> marshal(Map<K, V> value) throws Exception {
      InternalMapType<K, W> map = new InternalMapType<K, W>();
      for (Map.Entry<K, V> e : value.entrySet()) {
        if (valueAdapter == null) {
          map.getEntries().add(new InternalMapEntryType<K, W>(e.getKey(), (W) e.getValue()));
        } else {
          map.getEntries().add(new InternalMapEntryType<K, W>(e.getKey(), valueAdapter.marshal(e.getValue())));
        }
      }
      return map;
    }

  }

  /**
   * @author Guillaume Porcher
   */
  public static class JAXBMapDefinitionsAdapter extends JAXBMapAdapter<String, Set<Definition>, SimpleCollectionWrapper<String>> {
    public JAXBMapDefinitionsAdapter() {
      this.valueAdapter = new JAXBDefinitionsAdapter();
    }
  }

  /**
  * @author Guillaume Porcher
  */
  public static class JAXBMapDocumentAdapter extends JAXBMapAdapter<String, Document, String> {
    public JAXBMapDocumentAdapter() {
      this.valueAdapter = new JAXBDOMDocumentAdapter();
    }
  }

  /**
  * @author Guillaume Porcher
  */
  public static class JAXBMapNodeAdapter extends JAXBMapAdapter<String, Node, String> {
    public JAXBMapNodeAdapter() {
      this.valueAdapter = new JAXBDOMNodeAdapter();
    }
  }
}
